@@ -7,7 +7,8 @@
 #' @param dat matrix or data.frame with rows for multiple (independent) tests, use ONLY with 2 groups; assumed as log2-data !!!
 #' @param grp (factor) describes column-relationship of 'dat'   (1st factor is considered as reference -> orientation of M-values !!)
 #' @param limmaOutput (logical) return full (or extended) MArrayLM-object from limma or 'FAlSE' for only the (uncorrected) p.values
-#' @param addResults (character) types of results to add besides basic limma-output (eg "lfdr" using fdrtool-package,"FDR" for BY-FDR,"Mval" (assumes that data are log2 !),"means" or "nonMod" for non-moderated test)
+#' @param addResults (character) types of results to add besides basic limma-output, data are assumed to be log2 ! (eg "lfdr" using fdrtool-package, "FDR" or "BH" for BH-FDR, "BY" for BY-FDR, 
+#'   "bonferroni" for Bonferroni-correction, "qValue" for lfdr by qvalue, "Mval", "means" or "nonMod" for non-moderated test and he equivaent all (other) multiple testing corrections chosen here)
 #' @param testOrientation (character) for one-sided test (">","greater" or "<","less"), NOTE : 2nd grp is considered control/reference, '<' will identify grp1 < grp2
 #' @param silent (logical) suppress messages
 #' @param callFrom (character) allow easier tracking of message(s) produced
@@ -23,16 +24,24 @@
 #' ## expect to find C/A in c,d,g, (h)
 #' ## expect to find C/D in c,d,e,f
 #' ## expect to find A/D in f,g,(h)  
-#' test8 <- moderTestXgrp(t8,grp) 
-#' head(test8$p.value,n=8)
+#' test8 <- moderTestXgrp(t8, grp) 
+#' head(test8$p.value, n=8)
 #' @export
-moderTestXgrp <- function(dat,grp,limmaOutput=TRUE,addResults=c("lfdr","FDR","Mval","means"),testOrientation="=",silent=FALSE,callFrom=NULL){
+moderTestXgrp <- function(dat, grp, limmaOutput=TRUE, addResults=c("lfdr","FDR","Mval","means"), testOrientation="=", silent=FALSE, callFrom=NULL){
   fxNa <- .composeCallName(callFrom, newNa="moderTestXgrp")
-  chPa <- try(find.package("limma"), silent=TRUE)
-  if("try-error" %in% class(chPa)) stop("package 'limma' not found !") 
-  chFdr <- try(find.package("fdrtool"), silent=TRUE)  
+  if(any(length(dat) <1, length(dim(dat)) !=2, dim(dat) < c(1,3))) stop("Invalid argument 'dat'; must be matrix or data.frame with min 1 lines and 3 columns")
+  if(length(grp) != ncol(dat)) stop("Check parameters: Number of columns of 'dat' doesn't match to length of 'grp'")  
   if(!is.factor(grp)) grp <- as.factor(grp)
   if(length(levels(grp)) <2) stop(" need at least 2 groups in argument 'grp'")
+  chPa <- c(try(find.package("limma"), silent=TRUE), try(find.package("fdrtool"), silent=TRUE), try(find.package("qvalue"), silent=TRUE))
+  if("try-error" %in% class(chPa[1])) stop("package 'limma' not found ! Please install first from Bioconductor")
+  if(limmaOutput & length(addResults) >0) if("all" %in% addResults) addResults <- c("BH", "BY","lfdr","qValue","bonferroni","Mval","means","nonMod")
+  if("try-error" %in% class(chPa[2]) & any("lfdr" %in% tolower(addResults))) {
+    if(!silent) message(fxNa," package 'fdrtool' not found, omitting .. Please install from CRAN for enabeling 'lfdr'")
+    addResults <- addResults[which(!tolower(addResults) %in% "lfdr")] }
+  if("try-error" %in% class(chPa[3]) & any(c("qvalue","qval") %in% tolower(addResults))) {
+    if(!silent) message(fxNa," package 'qvalue' not found, omitting .. Please install from Bioconductor for enabeling 'qValue'")
+    addResults <- addResults[which(!tolower(addResults) %in% c("qvalue","qval"))] }  
   ## treat non-unique row-names ?
   if(length(rownames(dat) >0)) if(length(unique(rownames(dat))) < nrow(dat)) {
     if(!silent) message(fxNa," detected ",nrow(dat)-length(unique(rownames(dat)))," non-unique rownames of 'dat' !")}
@@ -40,9 +49,7 @@ moderTestXgrp <- function(dat,grp,limmaOutput=TRUE,addResults=c("lfdr","FDR","Mv
   if(length(testOrientation) <1) testOrientation <- altHyp
   if(testOrientation %in% c("<","less","inf")) altHyp <- "less"           
   if(testOrientation %in% c(">","greater","sup")) altHyp <- "greater"
-  if(length(addResults) >0) if("lfdr" %in% tolower(addResults) & "try-error" %in% class(chFdr)) {
-    if(!silent) message(fxNa,"package 'fdrtool' not found ! Please install package fdrtool from CRAN for enabeling 'lfdr' estimations") 
-    addResults <- addResults[which(!tolower(addResults) %in% "lfdr")] }  
+  ## prepare modeling 
   datDesign <- stats::model.matrix(~ -1 + grp)                  # can't use directly, need contrasts !!
   colnames(datDesign) <- sub("^grp","", colnames(datDesign))
   comp <- triCoord(length(levels(grp)))
@@ -50,8 +57,8 @@ moderTestXgrp <- function(dat,grp,limmaOutput=TRUE,addResults=c("lfdr","FDR","Mv
   contr.matr <- matrix(0, nrow=length(levels(grp)), ncol=nrow(comp), dimnames=list(levels(grp),rownames(comp)))
   for(j in 1:nrow(comp)) contr.matr[comp[j,],j] <- c(1,-1)
   ## see eg   https://support.bioconductor.org/p/57268/; https://www.biostars.org/p/157068/
-  globFilt <- 1:nrow(dat)  # not used yet
-  fit0 <- try(limma::lmFit(dat[globFilt,], datDesign))          # [sampleOrder,qcDat]
+  globFilt <- 1:nrow(dat)                                       # so far apply testing to all lines
+  fit0 <- try(limma::lmFit(dat[globFilt,], datDesign))          # testing part 1
   if("try-error" %in% class(fit0)) message(fxNa," Problem running lmFit(),  check if package 'limma' is installed !?!")
   fit1 <- limma::eBayes(limma::contrasts.fit(fit0, contrasts=contr.matr))  # variant to run all contrasts at same time
   compNa <- colnames(fit1$contrasts)
@@ -76,15 +83,27 @@ moderTestXgrp <- function(dat,grp,limmaOutput=TRUE,addResults=c("lfdr","FDR","Mv
   }
   if(is.null(colnames(fit1$t))) colnames(fit1$t) <- compNa
   if(is.null(colnames(fit1$p.value))) colnames(fit1$p.value) <- compNa
+  ## add various multiple testing corrections
   if(!limmaOutput) out <- fit1$p.value[,2] else { out <- fit1
     ## further inspect & correct values of 'addResults' ?
     if("Mval" %in% addResults) out$Mval <- (out$means[,1] - out$means[,2]) 
-    if("FDR" %in% toupper(addResults)) out$FDR <- if(is.matrix(out$p.value)) {
+    if(any(c("FDR","BH") %in% toupper(addResults))) out$FDR <- if(is.matrix(out$p.value)) {
+      apply(out$p.value, 2, stats::p.adjust, meth="BH")} else stats::p.adjust(out$p.value, meth="BH")
+    if("BY" %in% toupper(addResults)) out$BY <- if(is.matrix(out$p.value)) {
       apply(out$p.value, 2, stats::p.adjust, meth="BY")} else stats::p.adjust(out$p.value, meth="BY")
     if("lfdr" %in% tolower(addResults)) {out$lfdr <- if(is.matrix(out$p.value)) {
-      apply(out$p.value, 2, pVal2lfdr)} else pVal2lfdr(out$p.value)
-      }    
-    }
+      apply(out$p.value, 2, pVal2lfdr)} else pVal2lfdr(out$p.value) }    
+    if(any(c("qval","qvalue") %in% tolower(addResults))) { out$qVal <- if(is.matrix(out$p.value)) {
+      try(apply(out$p.value, 2, function(x) qvalue::qvalue(x,lfdr.out=TRUE)$lfdr), silent=TRUE)} else try(qvalue::qvalue(out$p.value,lfdr.out=TRUE)$lfdr, silent=TRUE) 
+      if("try-error" %in% class(out$qVal)) { 
+        if(!silent) message(fxNa," Problem with pi0 estimation, setting pi0=1 like BH-FDR")
+        out$qVal <- if(is.matrix(out$p.value)) { apply(out$p.value, 2, function(x) qvalue::qvalue(x, pi0=1, lfdr.out=TRUE)$lfdr)
+          } else qvalue::qvalue(out$p.value, pi0=1, lfdr.out=TRUE)$lfdr }
+    }    
+    if(any(c("bonferroni","bonf") %in% tolower(addResults))) out$bonf <- if(is.matrix(out$p.value)) {
+      apply(out$p.value, 2, stats::p.adjust, meth="bonferroni")} else stats::p.adjust(out$p.value, meth="bonferroni")
+  }
+  
   if("nonMod" %in% addResults) { leLev <- length(levels(grp))
     grX <- lapply(2:leLev, function(x) which(grp==levels(grp)[x]))
     grX[2:leLev] <- grX[1:(length(levels(grp)) -1)]
@@ -94,9 +113,30 @@ moderTestXgrp <- function(dat,grp,limmaOutput=TRUE,addResults=c("lfdr","FDR","Mv
     if(altHyp=="=") altHyp <- "two.sided"
     comp <- triCoord(leLev)
     out$nonMod.p <- apply(comp,1,function(y) apply(dat, 1, function(x) stats::t.test(x[which(grp==levels(grp)[y[1]])], x[which(grp==levels(grp)[y[2]])], alternative=altHyp)$p.value) )
-    out$nonMod.FDR <- if(length(dim(out$nonMod.p))==2) apply(out$nonMod.p,2,stats::p.adjust, method="BY") else stats::p.adjust(out$nonMod.p, method="BY")
-    out$nonMod.lfdr <- if(length(dim(out$nonMod.p))==2) apply(out$nonMod.p, 2, pVal2lfdr) else pVal2lfdr(out$nonMod.p)
-    colnames(out$nonMod.p) <- colnames(out$nonMod.FDR)  <- colnames(out$nonMod.lfdr) <- compNa 
+    colnames(out$nonMod.p) <- compNa
+    if(any(c("FDR","BH") %in% toupper(addResults))) {
+      if(length(dim(out$nonMod.p))==2) { out$nonMod.FDR <- apply(out$nonMod.p, 2, stats::p.adjust, method="BH")
+      colnames(out$nonMod.FDR)  <- compNa } else out$nonMod.FDR <- stats::p.adjust(out$nonMod.p, method="BH")
+      }
+    if(any("BY" %in% toupper(addResults))) {
+      if(length(dim(out$nonMod.p))==2) { out$nonMod.BY <- apply(out$nonMod.p, 2, stats::p.adjust, method="BY")
+      colnames(out$nonMod.BY)  <- compNa } else out$nonMod.BY <- stats::p.adjust(out$nonMod.p, method="BY")
+      }
+    if(any("lfdr" %in% tolower(addResults))) {
+      if(length(dim(out$nonMod.p))==2) { out$nonMod.lfdr <- apply(out$nonMod.p, 2, pVal2lfdr)
+      colnames(out$nonMod.lfdr)  <- compNa } else out$nonMod.lfdr <- pVal2lfdr(out$nonMod.p)
+      }
+    if(any(c("qval","qvalue") %in% tolower(addResults))) {
+      out$nonMod.qVal <- if(length(dim(out$nonMod.p))==2) try(apply(out$nonMod.p, 2, qvalue::qvalue), silent=TRUE) else try(qvalue::qvalue(out$nonMod.p), silent=TRUE)
+      if("try-error" %in% class(out$nonMod.qVal)) { if(!silent) message(fxNa," Problem with pi0 estimation (non-shrinked p-values) for qValue, setting pi0=1 like BH-FDR")
+        out$nonMod.qVal <- if(length(dim(out$nonMod.p))==2) apply(out$nonMod.p, 2, qvalue::qvalue,pi0=1, lfdr.out=TRUE) else qvalue::qvalue(out$nonMod.p,pi0=1, lfdr.out=TRUE)
+      }
+      if(length(dim(out$nonMod.p))==2) colnames(out$nonMod.qVal)  <- compNa 
+      }
+    if(any(c("bonferroni","bonf") %in% tolower(addResults))) {
+      if(length(dim(out$nonMod.p))==2) { out$nonMod.bonf <- apply(out$nonMod.p, 2, stats::p.adjust, method="bonferroni")
+      colnames(out$nonMod.bonf)  <- compNa } else out$nonMod.lfdr <- stats::p.adjust(out$nonMod.p, method="bonferroni")
+      }
     }
   out }  
    
